@@ -32,9 +32,7 @@ import os
 
 import parce
 import parce.lang
-from parce.action import StandardAction
-from parce.rule import DynamicItem
-from parce.lexicon import LexiconDescriptor, Lexicon
+from parce.action import Comment
 
 
 class Language:
@@ -55,53 +53,10 @@ class Language:
         Most bundled languages use this method for their comment lexicons.
 
         """
-        yield r'\b\w(?:[._%+-]?\w+)*@\w(?:[._-]?\w+)*\b', parce.Comment.Email
-        yield r'(?:https?|ftp):/(?:[\w._~:?/#-]+|\([\w._~:?/#-]*\))+', parce.Comment.Url
-        yield r"\b(ALERT|BUG|FIXME|TEMP|TODO|XXX+)\b", parce.Comment.Alert
-        yield parce.default_action, parce.Comment
-
-
-def lexicons(lang):
-    """Return a list of all the lexicons on the language."""
-    lexicons = []
-    for key, value in lang.__dict__.items():
-        if isinstance(value, LexiconDescriptor):
-            lexicons.append(getattr(lang, key))
-    return lexicons
-
-
-def rule_items(lang):
-    """Yield all rule items in a language, flattening all DynamicItem instances."""
-    def flatten(items):
-        for i in items:
-            if isinstance(i, DynamicItem):
-                for l in i.itemlists:
-                    yield from flatten(l)
-            else:
-                yield i
-    for lexicon in lexicons(lang):
-        for rule in lexicon:
-            yield from flatten(rule)
-
-
-def standardactions(lang):
-    """Return the set of all the StandardAction instances in the language.
-
-    Does not follow targets to other languages.
-
-    """
-    return set(i for i in rule_items(lang) if isinstance(i, StandardAction))
-
-
-def languages(lang):
-    """Return the set of all languages that this language refers to.
-
-    Does not follow targets from languages that are referred to.
-
-    """
-    return set(i.language
-        for i in rule_items(lang)
-            if isinstance(i, Lexicon) and i.language is not lang)
+        yield r'\b\w+(?:[._%+-]\w+)*@\w+(?:[._-]\w+)*\b', Comment.Email
+        yield r'(?:(?:https?|ftp):/|\bwww\.)(?:[\w_~:/#-]+([.?=][\w_~:/#-]+)*|\([\w._~:?/#-]*\))+', Comment.Url
+        yield r"\b(ALERT|BUG|FIXME|TEMP|TODO|XXX+)\b", Comment.Alert
+        yield parce.default_action, Comment
 
 
 def get_all_modules():
