@@ -49,6 +49,7 @@ See also the documentation for Token and Context.
 
 
 import itertools
+import reprlib
 
 from parce import util
 from parce import query
@@ -396,7 +397,7 @@ class Token(Node):
         return c1.parent is None and c2.parent is None
 
     def __repr__(self):
-        text = util.abbreviate_repr(self.text[:31])
+        text = reprlib.repr(self.text)
         return "<Token {} at {}:{} ({})>".format(text, self.pos, self.end, self.action)
 
     def __eq__(self, other):
@@ -938,3 +939,21 @@ class Context(list, Node):
                 token = token.group[0]
             return token
 
+
+def make_tokens(event, parent=None):
+    """Factory returning a tuple of one or more Token instances for the event.
+
+    The event is an Event namedtuple defined in the mod:`~parce.lexer` module.
+    If the event contains more than one token, _GroupToken instances are
+    created.
+
+    """
+    if len(event.tokens) > 1:
+        tokens = tuple(_GroupToken(parent, *t) for t in event.tokens)
+        for t in tokens:
+            t.group = tokens
+        return tokens
+    else:
+        return Token(parent, *event.tokens[0]),
+
+
