@@ -45,19 +45,20 @@ threads.
 
 """
 
-
+import operator
 import threading
 
 from parce.lexer import Lexer
-from parce.tree import Context, Token, _GroupToken
-from parce.util import tokens
+from parce.util import Observable, tokens
 from parce.target import TargetFactory
+from parce.tree import Context, make_tokens
 from parce.treebuilderutil import (
-    BuildResult, ReplaceResult, Changes, find_insert_tokens, get_lexer, new_tree)
+    BuildResult, ReplaceResult, Changes, get_prepared_lexer, new_tree)
 
 
 def build_tree(root_lexicon, text, pos=0):
     """Build and return a tree in one go."""
+    from parce.tree import Context, make_tokens # local is faster
     root = context = Context(root_lexicon, None)
     lexer = Lexer([root_lexicon])
     for e in lexer.events(text, pos):
@@ -67,17 +68,11 @@ def build_tree(root_lexicon, text, pos=0):
             for lexicon in e.target.push:
                 context = Context(lexicon, context)
                 context.parent.append(context)
-        if len(e.tokens) > 1:
-            tokens = tuple(_GroupToken(context, *t) for t in e.tokens)
-            for t in tokens:
-                t.group = tokens
-            context.extend(tokens)
-        else:
-            context.append(Token(context, *e.tokens[0]))
+        context.extend(make_tokens(e, context))
     return root
 
 
-class TreeBuilder:
+class TreeBuilder(Observable):
     """Build a tree from parsing the text.
 
     The root node of the tree is in the ``root`` instance attribute.
@@ -113,6 +108,51 @@ class TreeBuilder:
     No other variables or state are kept, so if you don't need the above
     information anymore, you can throw away the TreeBuilder after use.
 
+    During the building process, the TreeBuilder emits certain events you can
+    subscribe to, using the :meth:`~parce.util.Observable.connect` method
+    provided by the :class:`~parce.util.Observable` class that's mixed into
+    this TreeBuilder class.
+
+    The following events are emitted, with following arguments:
+
+    ``"started"``:
+        emitted when a (re)build starts; the handler is called without
+        arguments
+
+    ``"replace"``:
+        emitted just before the tree actually changes (while the new tree is
+        being built, the tree is still unchanged and accessible, but between
+        the ``"replace"`` and ``"finished"`` events the tree is in an
+        inconsistent state)
+
+    ``"finished"``:
+        emitted when a (re)build has finished; the handler is called without
+        arguments
+
+    ``"updated"``:
+        emitted when a (re)build has finished; the handler is called with two
+        arguments: ``start``, ``end``, that denote the updated range
+
+    ``"peek"``:
+        emitted by the default implementation of the :meth:`peek` method,
+        the handler is called with two arguments: ``start``, ``tree``
+
+    ``"invalidate"``:
+        emitted by the default implementation of the :meth:`invalidate_context`
+        method, the handler is called with the Context that needs to be
+        invalidated
+
+    For example, to get notified when a build process starts::
+
+        >>> b = TreeBuilder(MyLang.root)
+        >>> def hi_there():
+        ...     print("started")
+        ...
+        >>> b.connect("started", hi_there)
+        >>> b.rebuild("some boring text")
+        started
+        >>>
+
     """
     start = 0
     end = 0
@@ -121,6 +161,7 @@ class TreeBuilder:
     peek_threshold = 0  #: set to a value > 0 to get :meth:`peek` called during building
 
     def __init__(self, root_lexicon=None):
+        super().__init__()
         self.root = Context(root_lexicon, None)
         self.busy = False
         self.changes = []
@@ -211,6 +252,8 @@ class TreeBuilder:
         gives the position change for the tokens that are reused.
 
         """
+        from parce.tree import Context, make_tokens # local is faster
+
         if root_lexicon is not False:
             start, removed, added = 0, 0, len(text)
         else:
@@ -244,12 +287,11 @@ class TreeBuilder:
         while True:
             # when restarting, see if we can reuse (part of) the new tree
             if tree:
-                tokens = find_insert_tokens(tree, text, start)
-                if tokens:
+                result = get_prepared_lexer(tree, text, start)
+                if result:
+                    lexer, events, tokens = result
                     t = tokens[0]
                     context = t.parent
-                    lexer = get_lexer(t)
-                    events = lexer.events(text, t.pos)
                     for p, i in t.ancestors_with_index():
                         del p[i+1:]
                     del context[-1]
@@ -258,13 +300,11 @@ class TreeBuilder:
             # find insertion spot in old tree
             if not tree:
                 start = min(lowest_start, start)
-                tokens = find_insert_tokens(self.root, text, start)
-                if tokens:
+                result = get_prepared_lexer(self.root, text, start)
+                if result:
+                    lexer, events, tokens = result
                     t = tokens[0]
                     context = new_tree(t)
-                    lexer = get_lexer(t)
-                    events = lexer.events(text, t.pos)
-                    next(events) # skip over the first token, we need its target
                     tree = context.root()
                     start = tokens[-1].end
                     lowest_start = min(lowest_start, start)
@@ -282,12 +322,7 @@ class TreeBuilder:
                     for lexicon in e.target.push:
                         context = Context(lexicon, context)
                         context.parent.append(context)
-                if len(e.tokens) > 1:
-                    tokens = tuple(_GroupToken(context, *t) for t in e.tokens)
-                    for t in tokens:
-                        t.group = tokens
-                else:
-                    tokens = Token(context, *e.tokens[0]),
+                tokens = make_tokens(e, context)
                 if tail:
                     # handle tail
                     pos = tokens[0].pos - offset
@@ -359,6 +394,9 @@ class TreeBuilder:
         can be reimplemented to get fine-grained monitoring of and control over
         the tree-replacing process.
 
+        Additionally, this method calls :meth:`invalidate_context` with the
+        youghest Context that had children removed or added.
+
         """
         tree, start, end, offset, lexicons = result
 
@@ -369,6 +407,7 @@ class TreeBuilder:
                 n.parent = root
             self.replace_nodes(self.root, slice(None), tree)
             self.replace_root_lexicon(tree.lexicon)
+            self.invalidate_context(self.root)
 
         else:
 
@@ -429,8 +468,10 @@ class TreeBuilder:
                 for n in t:
                     n.parent = c
                 self.replace_nodes(c, slice(i + 1, slice_end), t)
+                self.invalidate_context(c)
             else:
                 self.replace_nodes(context, slice(slice_end), tree)
+                self.invalidate_context(context)
 
             if offset:
                 for p, i in context.ancestors_with_index():
@@ -466,18 +507,46 @@ class TreeBuilder:
         for t in tokens(context[slice_]):
             t.pos += offset
 
-    def get_root(self, wait=False, *args, **kwargs):
+    def invalidate_context(self, context):
+        """Called with the younghest Context that had children are removed or
+        added.
+
+        This means that the meaning of this context probably has changed, for
+        example when you want to transform the context to some other data
+        structure, and that the ancestors also need to be invalidated.
+
+        The default implementation of this method emits the ``invalidate``
+        event, see :meth:`~parce.util.Observable.connect`.
+
+        """
+        self.emit("invalidate", context)
+
+    def get_root(self, wait=False, callback=None, args=(), kwargs={}):
         """Return the root element of the completed tree.
 
         This is simply the ``root`` instance attribute, but this method only
         returns the tree when the ``busy`` attribute is False.
 
-        If tree building is busy, returns None or calls :meth:`wait` if
-        ``wait`` is True. The default implementation ignores the other
-        arguments.
+        If wait is True, this call blocks until tokenizing is done, and the
+        full tree is returned. If wait is False, None is returned if the tree
+        is still busy being built.
+
+        If a callback is given and tokenizing is still busy, that callback is
+        called once when tokenizing is ready. If given, args and kwargs are the
+        arguments the callback is called with, defaulting to () and {},
+        respectively.
+
+        Note that, for the lifetime of a TreeBuilder, the root element is always
+        the same. The root element is also accessible in the `root` attribute.
+        But using this method you can be sure that you are dealing with a
+        complete and fully intact tree.
 
         """
         if self.busy:
+            if callback:
+                if args or kwargs:
+                    callback = lambda: callback(*args, **kwargs)
+                self.connect("finished", callback, True)
             if not wait:
                 return
             self.wait()
@@ -534,6 +603,7 @@ class TreeBuilder:
             yield "build"
             result = self.build_new_tree(c.text, c.root_lexicon, c.start, c.removed, c.added)
             yield "replace"
+            self.emit("replace")
             r = self.replace_tree(result)
             start = r.start if start == -1 else min (start, r.start)
             end = r.end if end == -1 else max(c.new_position(end), r.end)
@@ -583,11 +653,11 @@ class TreeBuilder:
         peek() to be called a second time. (A build is restarted when there are
         new changes close to the position the build originally started.)
 
-        The default implementation of this method does nothing; the default
-        value of the  ``peek_threshold`` attribute is 0.
+        The default implementation of this method emits the ``peek`` event, see
+        :meth:`~parce.util.Observable.connect`.
 
         """
-        pass
+        self.emit("peek", start, tree)
 
     def lock(self, acquire):
         """Acquire lock (True) or release lock (False). Does nothing by default.
@@ -601,18 +671,22 @@ class TreeBuilder:
     def process_started(self):
         """Called at the start ot the tree building process.
 
-        Does nothing by default.
+        The default implementation of this method emits the ``started`` event,
+        see :meth:`~parce.util.Observable.connect`.
 
         """
-        pass
+        self.emit("started")
 
     def process_finished(self):
         """Called when tree building is done.
 
-        Does nothing by default.
+        The default implementation of this method emits the ``updated(start,
+        end)`` and ``finished`` events, see
+        :meth:`~parce.util.Observable.connect`.
 
         """
-        pass
+        self.emit("updated", self.start, self.end)
+        self.emit("finished")
 
 
 class BackgroundTreeBuilder(TreeBuilder):
@@ -625,14 +699,6 @@ class BackgroundTreeBuilder(TreeBuilder):
     You can continue adding changes while previous changes are processed;
     the tree builder will immediately adapt to the new changes.
 
-    You can add callbacks to the ``updated_callbacks`` attribute (using
-    ``add_build_updated_callback()``) that are called everytime the whole
-    document is tokenized.
-
-    You can also add callbacks to the ``finished_callbacks`` attribute, using
-    ``add_finished_callback()``; those are called once when all pending changes
-    are processed and then forgotten again.
-
     To be sure you get a completed tree, call ``get_root(True)``.
 
     """
@@ -640,8 +706,6 @@ class BackgroundTreeBuilder(TreeBuilder):
         super().__init__(root_lexicon)
         self.job = None
         self._lock = threading.Lock()
-        self.updated_callbacks = []
-        self.finished_callbacks = []
 
     def lock(self, acquire):
         """Reimplemented to actually lock/unlock."""
@@ -652,84 +716,15 @@ class BackgroundTreeBuilder(TreeBuilder):
         self.job = threading.Thread(target=super().start_processing)
         self.job.start()
 
-    def process_finished(self):
-        """Reimplemented to clear the job attribute and call the callbacks."""
-        self.job = None
-        for cb in self.updated_callbacks:
-            cb(self.start, self.end)
-        while self.finished_callbacks:
-            callback, args, kwargs = self.finished_callbacks.pop()
-            callback(*args, **kwargs)
-
     def wait(self):
         """Reimplemented to await our background thread if active."""
         job = self.job
         if job:
             job.join()
 
-    def get_root(self, wait=False, callback=None, args=None, kwargs=None):
-        """Get the root element of the completed tree.
+    def process_finished(self):
+        """Reimplemented to clear the job attribute."""
+        self.job = None
+        super().process_finished()
 
-        If wait is True, this call blocks until tokenizing is done, and the
-        full tree is returned. If wait is False, None is returned if the tree
-        is still busy being built.
-
-        If a callback is given and tokenizing is still busy, that callback is
-        called once when tokenizing is ready. If given, args and kwargs are the
-        arguments the callback is called with, defaulting to () and {},
-        respectively.
-
-        Note that, for the lifetime of a TreeBuilder, the root element is always
-        the same. The root element is also accessible in the `root` attribute.
-        But using this method you can be sure that you are dealing with a
-        complete and fully intact tree.
-
-        """
-        if not self.job:
-            return self.root
-        if callback:
-            self.add_finished_callback(callback, args, kwargs)
-        if wait:
-            self.wait()
-            return self.root
-
-    def add_build_updated_callback(self, callback):
-        """Add a callback to be called when the whole text is tokenized.
-
-        The callback is called with two arguments (start, end) denoting
-        the range in the text that was tokenized again.
-
-        """
-        if callback not in self.updated_callbacks:
-            self.updated_callbacks.append(callback)
-
-    def remove_build_updated_callback(self, callback):
-        """Remove a previously registered callback to be called when the whole text is tokenized."""
-        if callback in self.updated_callbacks:
-            self.updated_callbacks.remove(callback)
-
-    def add_finished_callback(self, callback, args=None, kwargs=None):
-        """Add a callback to be called when tokenizing finishes.
-
-        This callback will be called once, directly after being called
-        it will be forgotten.
-
-        """
-        if args is None:
-            args = ()
-        if kwargs is None:
-            kwargs = {}
-        cb = (callback, args, kwargs)
-        if cb not in self.finished_callbacks:
-            self.finished_callbacks.append(cb)
-
-    def remove_finished_callback(self, callback, args=None, kwargs=None):
-        """Remove a callback that was registered to be called when tokenizing finishes."""
-        if args is None:
-            args = ()
-        if kwargs is None:
-            kwargs = {}
-        cb = (callback, args, kwargs)
-        if cb in self.finished_callbacks:
-            self.finished_callbacks.remove(cb)
 
